koliseo-widgets
===============

A collection of widgets that use the Koliseo API. You can see a live example <a href="http://icoloma.github.io/koliseo-widgets/index.html">here</a>.

These widgets can be used to sell tickets for your events from your own pages without using iframes. They are responsive and have been tested with <b>Chrome, Firefox and IE8+</b>. Older browsers will degrade to a simple "get your tickets" button.

KPerf
=====

KPerf displays a list of scheduled performances for a event. 

<img src="screenshot.png" alt="Screenshot of the KPerf widget">

To use, add this at the end of your web page (before the ending <code>&lt;/body></code> tag):

```html
<!-- KPerf requires jQuery. If your web page already has jQuery present, you may omit this first line. -->
<script src="//ajax.googleapis.com/ajax/libs/jquery/1.7.2/jquery.min.js"></script>
<script src="http://www.koliseo.com/w/kperf/1.0/kperf.min.js"></script>
```

Now put this where you want the widget to appear:

```html
<div class="kcontainer" data-event-uuid="{{{your event UUID}}}"></div>
```

Parameters
==========

KPerf can receive parameters as <code>data-*</code> attributes of the container DOM element:

* **eventUuid** (string) the event uuid, extracted from a Koliseo URL such as <code>sandokan/board-her</code>. You can specify multiple values separated by commas (no spaces). *This attribute is required*.
* **locale** (string) the two-letter acronym of the locale to use. The default is the browser locale or <code>en</code> if none.
* **dateFormat** (string) the date format. The default is locale-specific.
* **maxEntries** (number) the maximum number of entries to display. The default is 5.
* **insertCss** (boolean) By default <code>kperf.css</code> will be inserted to <code>&lt;head></code> if it is not already present. Set this to false if you plan to provide your own styles somewhere else.

Notice that any camelcase parameter names should be introduced in your HTML using dashes, as in <code>data-event-uuid</code> (instead of <code>data-eventUuid</code>). Only the event uuid is required. An example using all parameters:

```html
<div class="kcontainer" 
  data-event-uuid="john-doe/bacon-ipsum"
  data-locale="es"
  data-date-format="Ddd Mm* d*"
  data-max-entries="10"
  data-insert-css="false"
></div>
```

Using KPerf as a jQuery plugin
==============================

KPerf is also a jQuery plugin that can also be initialized using JavaScript:

```javascript
$('#my-container').kperf({
  eventUuid: 'solid-snake/destroy-mg',
  locale: 'es',
  dateFormat: $('#my-container').width() < 200? 'dd/mm' : 'Ddd Mm* d*'
});
```

Locales
=======

The widget is automatically set to use the browser locale, unless otherwise specified using the <code>locale</code> option. The widget supports the same set of locales as the Koliseo website, and will default to <code>en</code> if a locale is not supported.

You can add new locales by setting a global Koliseo.resources variable before including <code>kperf.js</code>:

```javascript
window.Koliseo = window.Koliseo || {};
Koliseo.resources = {
  it: {
    ticketsAvailable: 'biglietti disponibili',
    seeOther: 'vedi altre date',
    buy: 'Compra',
    empty: 'Nessuno spettacolo trovato',
    dateFormat: 'Ddd Mm* d*',
    weekDays: 'Domenica Lunedì Martedì Mercoledì Giovedì Venerdì Sabato',
    months: 'Gennaio Febbraio Marzo Aprile Maggio Giugno Luglio Agosto Settembre Ottobre Novembre Dicembre'
  }
};
```

License
=======

These widgets are released under the [MIT license](http://en.wikipedia.org/wiki/MIT_License). You may modify these widgets and include them in your own commercial, closed-source projects as long as the copyright header is left intact.
