
var CalendarView;

(function() {
  // Extract UTC YYYY-MM-DD date string from param @date (parsed as a Date)
  var extractDateStr = function(date) {
    var dateStr = (new Date(date)).toISOString();
    return (/(.+)T/).exec(dateStr)[1];
  },

  // Extract UTC hh:mm time string from param @date (parsed as Date)
  extractTime = function(date) {
    var dateStr = (new Date(date)).toISOString();
    return (/T([0-9]+:[0-9]+)/).exec(dateStr)[1];
  },

  // Returns a new Date object with the same UTC date as @date
  // and UTC time 00:00.
  bareDate = function(date) {
    var dateStr = (new Date(date)).toISOString();
    return new Date((/(.+)T/).exec(dateStr)[1]);
  },

  // Finds the total number of days in the UTC month given by @date
  daysInMonth = function(date) {
    var lastDay = 28
    , copy = new Date(date.valueOf()) 
    , month = copy.getUTCMonth()
    ;

    copy.setUTCDate(lastDay);
    while (copy.getUTCMonth() === month) {
      lastDay++;
      copy.setUTCDate(lastDay);
    }
    return lastDay-1;
  }
  ;
  /*
    options:
      data (required): 
        An array of objects containing the data to be displayed. Each entry
        must contain the exact time with dateTime and a date-time string
        in ISO format 'YYYY-DD-HHThh:mm' with the *local* time
      $el: the element to be attached
      locale: hash with locale strings

  */

  CalendarView = function(options) {
    this.options = options;
    this.options.now = new Date();
    this.initialize();
  };

  CalendarView.prototype = {
    initialize: function() {
      var data = this.options.data
      ;

      // Sort data
      data.sort(function(a, b) {
        var aDate = new Date(a.dateTimeStr)
        , bDate = new Date(b.dateTimeStr);
        return aDate > bDate ? 1 : aDate < bDate ? -1 : 0;
      });

      this.$el = this.options.$el;
    },

    headerTmpl: function(dateStr) {
      return '<div class="kcontrols">' +
             '<a class="knext kicon-right-open" tabindex="0"><span class="kvisually-hidden">Next</span></a>' +
             '<a class="kprevious kicon-left-open" tabindex="0"><span class="kvisually-hidden">Previous</span></a>' +
             '<div class="kdate">' + dateStr + '</div>' +
             '</div>';
    },

    renderMonth: function($container, date) {
      var data = this.options.data 
      , startDate
      , startDateStr
      , monthLength = daysInMonth(date)
      , weekStart
      , dayData
      , today = bareDate(this.options.now)
      , $month
      , monthNames = this.options.locale.months.split(' ')
      , cursor = 0
      ;

      // Initialize starting date
      startDate = bareDate(date);
      startDate.setUTCDate(1);

      // Define containers
      $container.append(
        this.headerTmpl(monthNames[startDate.getUTCMonth()] + ' ' + startDate.getFullYear())
      );
      $month = $('<div class="kmonth"></div>').data('month', new Date(startDate));

      // Offset days
      weekStart = (startDate.getUTCDay() - this.options.locale.weekStart) % 7;
      for (var i = 0; i < weekStart; i++) {
        $month.append($('<div class="koffsetday"></div>'));
      }

      // Initialize cursor
      // If no initial entry is found, then cursor will be out of range
      for (i = 0; i < data.length; i++) {
        if (bareDate(extractDateStr(data[i].dateTimeStr)) >= startDate) {
          cursor = i;
          break;
        }
      }

      for (i = 1; i <= monthLength; i++) {
        startDate.setUTCDate(i);
        startDateStr = extractDateStr(startDate);

        // Days previous to current date
        if (today > startDate) {
          $month.append($('<span class="kpast">' + i + '</span>'));
          continue;
        }

        // Find data for current day and append
        dayData = [];
        while (cursor < data.length &&
          extractDateStr(data[cursor].dateTimeStr) === startDateStr) {
          dayData.push(data[cursor]);
          cursor++;
        }

        if (dayData.length) {
          $month.append(
            $('<span class="kday kperfday" tabindex="0">' + i + '</span>').data('performances', dayData)
          );
        } else {
          $month.append($('<span class="kday">' + i + '</span>'));
        }
      }

      $container.append($month);

    },

    changeMonth: function(direction) {
      var date = this.$el.find('.kmonth').data('month')
      ;
      date.setUTCMonth(date.getUTCMonth() + (direction > 0 ? 1 : -1));
      this.$el.empty();
      this.renderMonth(this.$el, date);
    },

    render: function() {
      var data = this.options.data
      ;

      this.renderMonth(this.$el, (data.length && data[0].dateTime) || new Date().valueOf());

      // Arrow handlers
      this.$el.on('click keydown', '.knext, .kprevious', {ctx: this}, function(e) {
        if (e.keyCode && e.keyCode !== 13) return;

        var view = e.data.ctx;

        if ($(this).hasClass('knext')) {
          view.changeMonth(1); 
          e.keyCode && view.$el.find('.knext')[0].focus();
        } else {
          view.changeMonth(-1); 
          e.keyCode && view.$el.find('.kprevious')[0].focus();
        }


      });

      return this;
    }
  };
})();